@@ -48,6 +48,7 @@
 #include "DFGOSREntry.h"
 #include "DFGOSRExit.h"
 #include "DFGVariableEventStream.h"
+#include "DeferredCompilationCallback.h"
 #include "EvalCodeCache.h"
 #include "ExecutionCounter.h"
 #include "ExpressionRangeInfo.h"
@@ -202,6 +203,8 @@ public:
 
     unsigned bytecodeOffset(ExecState*, ReturnAddressPtr);
 
+    void unlinkIncomingCalls();
+
 #if ENABLE(JIT)
     unsigned bytecodeOffsetForCallAtIndex(unsigned index)
     {
@@ -231,8 +234,6 @@ public:
     void linkIncomingCall(ExecState* callerFrame, LLIntCallLinkInfo*);
 #endif // ENABLE(LLINT)
 
-    void unlinkIncomingCalls();
-
 #if ENABLE(DFG_JIT) || ENABLE(LLINT)
     void setJITCodeMap(PassOwnPtr<CompactJITCodeMap> jitCodeMap)
     {
@@ -264,7 +265,38 @@ public:
 
     int argumentIndexAfterCapture(size_t argument);
 
-#if ENABLE(JIT)
+    // Prepares this code block for execution. This is synchronous. This compile
+    // may fail, if you passed JITCompilationCanFail.
+    CompilationResult prepareForExecution(
+        ExecState*, JITCode::JITType,
+        JITCompilationEffort = JITCompilationMustSucceed,
+        unsigned bytecodeIndex = UINT_MAX);
+    
+    // Use this method for asynchronous compiles. This will do a compile at some
+    // point in time between when you called into this method and some point in the
+    // future. If you're lucky then it might complete before this method returns.
+    // Once it completes, the callback is called with the result. If the compile
+    // did happen to complete before the method returns, the result of the compile
+    // may be returned. If the compile didn't happen to complete yet, or if we
+    // didn't happen to notice that the compile already completed, we return
+    // CompilationDeferred.
+    //
+    // Note that asynchronous compiles don't actually complete unless you call into
+    // DFG::Worklist::completeAllReadyPlansForVM(). You usually force a call to
+    // this on the main thread by listening to the callback's
+    // compilationDidBecomeReadyAsynchronously() notification. Note that this call
+    // happens on another thread.
+    CompilationResult prepareForExecutionAsynchronously(
+        ExecState*, JITCode::JITType, PassRefPtr<DeferredCompilationCallback>,
+        JITCompilationEffort = JITCompilationMustSucceed,
+        unsigned bytecodeIndex = UINT_MAX);
+    
+    // Exactly equivalent to codeBlock->ownerExecutable()->installCode(codeBlock);
+    void install();
+    
+    // Exactly equivalent to codeBlock->ownerExecutable()->newReplacementCodeBlockFor(codeBlock->specializationKind())
+    PassRefPtr<CodeBlock> newReplacement();
+    
     void setJITCode(PassRefPtr<JITCode> code, MacroAssemblerCodePtr codeWithArityCheck)
     {
         ConcurrentJITLocker locker(m_lock);
@@ -286,23 +318,14 @@ public:
         WTF::loadLoadFence(); // This probably isn't needed. Oh well, paranoia is good.
         return result;
     }
+
+#if ENABLE(JIT)
     bool hasBaselineJITProfiling() const
     {
         return jitType() == JITCode::BaselineJIT;
     }
-#if ENABLE(DFG_JIT)
-    virtual JSObject* compileOptimized(ExecState*, JSScope*, CompilationResult&, unsigned bytecodeIndex) = 0;
-    virtual CompilationResult replaceWithDeferredOptimizedCode(PassRefPtr<DFG::Plan>) = 0;
-#endif // ENABLE(DFG_JIT)
     void jettison();
-    CompilationResult jitCompile(ExecState* exec)
-    {
-        if (jitType() != JITCode::InterpreterThunk) {
-            ASSERT(jitType() == JITCode::BaselineJIT);
-            return CompilationNotNeeded;
-        }
-        return jitCompileImpl(exec);
-    }
+    
     virtual CodeBlock* replacement() = 0;
 
     virtual DFG::CapabilityLevel capabilityLevelInternal() = 0;
@@ -315,8 +338,6 @@ public:
     DFG::CapabilityLevel capabilityLevelState() { return m_capabilityLevelState; }
 
     bool hasOptimizedReplacement();
-#else
-    JITCode::JITType jitType() const { return JITCode::InterpreterThunk; }
 #endif
 
     ScriptExecutable* ownerExecutable() const { return m_ownerExecutable.get(); }
@@ -969,7 +990,6 @@ public:
     
 protected:
 #if ENABLE(JIT)
-    virtual CompilationResult jitCompileImpl(ExecState*) = 0;
     virtual void jettisonImpl() = 0;
 #endif
     virtual void visitWeakReferences(SlotVisitor&);
@@ -984,6 +1004,10 @@ protected:
 private:
     friend class DFGCodeBlocks;
     
+    CompilationResult prepareForExecutionImpl(
+        ExecState*, JITCode::JITType, JITCompilationEffort, unsigned bytecodeIndex,
+        PassRefPtr<DeferredCompilationCallback>);
+    
     void noticeIncomingCall(ExecState* callerFrame);
     
     double optimizationThresholdScalingFactor();
@@ -1085,12 +1109,12 @@ private:
     SegmentedVector<LLIntCallLinkInfo, 8> m_llintCallLinkInfos;
     SentinelLinkedList<LLIntCallLinkInfo, BasicRawSentinelNode<LLIntCallLinkInfo> > m_incomingLLIntCalls;
 #endif
+    RefPtr<JITCode> m_jitCode;
+    MacroAssemblerCodePtr m_jitCodeWithArityCheck;
 #if ENABLE(JIT)
     Vector<StructureStubInfo> m_structureStubInfos;
     Vector<ByValInfo> m_byValInfos;
     Vector<CallLinkInfo> m_callLinkInfos;
-    RefPtr<JITCode> m_jitCode;
-    MacroAssemblerCodePtr m_jitCodeWithArityCheck;
     SentinelLinkedList<CallLinkInfo, BasicRawSentinelNode<CallLinkInfo> > m_incomingCalls;
 #endif
 #if ENABLE(DFG_JIT) || ENABLE(LLINT)
@@ -1194,13 +1218,7 @@ public:
 
 #if ENABLE(JIT)
 protected:
-#if ENABLE(DFG_JIT)
-    virtual JSObject* compileOptimized(ExecState*, JSScope*, CompilationResult&, unsigned bytecodeIndex);
-    virtual CompilationResult replaceWithDeferredOptimizedCode(PassRefPtr<DFG::Plan>);
-#endif // ENABLE(DFG_JIT)
-
     virtual void jettisonImpl();
-    virtual CompilationResult jitCompileImpl(ExecState*);
     virtual CodeBlock* replacement();
     virtual DFG::CapabilityLevel capabilityLevelInternal();
 #endif
@@ -1223,13 +1241,7 @@ public:
     
 #if ENABLE(JIT)
 protected:
-#if ENABLE(DFG_JIT)
-    virtual JSObject* compileOptimized(ExecState*, JSScope*, CompilationResult&, unsigned bytecodeIndex);
-    virtual CompilationResult replaceWithDeferredOptimizedCode(PassRefPtr<DFG::Plan>);
-#endif // ENABLE(DFG_JIT)
-
     virtual void jettisonImpl();
-    virtual CompilationResult jitCompileImpl(ExecState*);
     virtual CodeBlock* replacement();
     virtual DFG::CapabilityLevel capabilityLevelInternal();
 #endif
@@ -1252,13 +1264,7 @@ public:
     
 #if ENABLE(JIT)
 protected:
-#if ENABLE(DFG_JIT)
-    virtual JSObject* compileOptimized(ExecState*, JSScope*, CompilationResult&, unsigned bytecodeIndex);
-    virtual CompilationResult replaceWithDeferredOptimizedCode(PassRefPtr<DFG::Plan>);
-#endif // ENABLE(DFG_JIT)
-
     virtual void jettisonImpl();
-    virtual CompilationResult jitCompileImpl(ExecState*);
     virtual CodeBlock* replacement();
     virtual DFG::CapabilityLevel capabilityLevelInternal();
 #endif
