@@ -40,7 +40,6 @@
 #include "FrameNetworkingContextGtk.h"
 #include "FrameTree.h"
 #include "FrameView.h"
-#include "GtkAuthenticationDialog.h"
 #include "GtkPluginWidget.h"
 #include "GtkUtilities.h"
 #include "HTMLAppletElement.h"
@@ -53,11 +52,11 @@
 #include "JSDOMWindow.h"
 #include "Language.h"
 #include "MIMETypeRegistry.h"
-#include "MainResourceLoader.h"
 #include "MouseEvent.h"
 #include "NotImplemented.h"
 #include "Page.h"
 #include "PluginDatabase.h"
+#include "PluginView.h"
 #include "ProgressTracker.h"
 #include "RenderPart.h"
 #include "RenderView.h"
@@ -66,6 +65,7 @@
 #include "ResourceRequest.h"
 #include "ScriptController.h"
 #include "Settings.h"
+#include "webkitauthenticationdialog.h"
 #include "webkiterror.h"
 #include "webkitfavicondatabase.h"
 #include "webkitfavicondatabaseprivate.h"
@@ -111,7 +111,6 @@ FrameLoaderClient::FrameLoaderClient(WebKitWebFrame* frame)
     , m_pluginView(0)
     , m_hasSentResponseToPlugin(false)
 {
-    ASSERT(m_frame);
 }
 
 FrameLoaderClient::~FrameLoaderClient()
@@ -156,7 +155,7 @@ void FrameLoaderClient::dispatchWillSubmitForm(FramePolicyFunction policyFunctio
     ASSERT(policyFunction);
     if (!policyFunction)
         return;
-    (core(m_frame)->loader()->policyChecker()->*policyFunction)(PolicyUse);
+    (core(m_frame)->loader().policyChecker()->*policyFunction)(PolicyUse);
 }
 
 void FrameLoaderClient::committedLoad(WebCore::DocumentLoader* loader, const char* data, int length)
@@ -167,7 +166,7 @@ void FrameLoaderClient::committedLoad(WebCore::DocumentLoader* loader, const cha
 
         Frame* coreFrame = loader->frame();
         if (coreFrame && coreFrame->document()->isMediaDocument())
-            loader->cancelMainResourceLoad(coreFrame->loader()->client()->pluginWillHandleLoadError(loader->response()));
+            loader->cancelMainResourceLoad(coreFrame->loader().client().pluginWillHandleLoadError(loader->response()));
     }
 
     if (m_pluginView) {
@@ -191,12 +190,23 @@ bool FrameLoaderClient::shouldUseCredentialStorage(WebCore::DocumentLoader*, uns
     return true;
 }
 
+// We convert this to string because it's easier to use strings as
+// keys in a GHashTable.
+static char* toString(unsigned long identifier)
+{
+    return g_strdup_printf("%ld", identifier);
+}
+
 void FrameLoaderClient::dispatchDidReceiveAuthenticationChallenge(WebCore::DocumentLoader*, unsigned long  identifier, const AuthenticationChallenge& challenge)
 {
+    WebKitWebView* view = webkit_web_frame_get_web_view(m_frame);
+
     if (DumpRenderTreeSupportGtk::dumpRenderTreeModeEnabled()) {
         CString username;
         CString password;
-        if (!DumpRenderTreeSupportGtk::s_authenticationCallback || !DumpRenderTreeSupportGtk::s_authenticationCallback(username, password)) {
+        GOwnPtr<gchar> identifierString(toString(identifier));
+        WebKitWebResource* webResource = webkit_web_view_get_resource(view, identifierString.get());
+        if (!DumpRenderTreeSupportGtk::s_authenticationCallback || !DumpRenderTreeSupportGtk::s_authenticationCallback(username, password, webResource)) {
             challenge.authenticationClient()->receivedRequestToContinueWithoutCredential(challenge);
             return;
         }
@@ -205,18 +215,16 @@ void FrameLoaderClient::dispatchDidReceiveAuthenticationChallenge(WebCore::Docum
         return;
     }
 
-    WebKitWebView* view = webkit_web_frame_get_web_view(m_frame);
-    GtkAuthenticationDialog::CredentialStorageMode credentialStorageMode;
 
-    if (core(view)->settings()->privateBrowsingEnabled())
-        credentialStorageMode = GtkAuthenticationDialog::DisallowPersistentStorage;
+    CredentialStorageMode credentialStorageMode;
+    if (core(view)->settings().privateBrowsingEnabled())
+        credentialStorageMode = DisallowPersistentStorage;
     else
-        credentialStorageMode = GtkAuthenticationDialog::AllowPersistentStorage;
+        credentialStorageMode = AllowPersistentStorage;
 
     GtkWidget* toplevel = gtk_widget_get_toplevel(GTK_WIDGET(view));
-    GtkWindow* toplevelWindow = widgetIsOnscreenToplevelWindow(toplevel) ? GTK_WINDOW(toplevel) : 0;
-    GtkAuthenticationDialog* dialog = new GtkAuthenticationDialog(toplevelWindow, challenge, credentialStorageMode);
-    dialog->show();
+    GtkWidget* authDialog = createAuthenticationDialog(widgetIsOnscreenToplevelWindow(toplevel) ? GTK_WINDOW(toplevel) : 0, challenge, credentialStorageMode);
+    gtk_widget_show(authDialog);
 }
 
 void FrameLoaderClient::dispatchDidCancelAuthenticationChallenge(WebCore::DocumentLoader*, unsigned long  identifier, const AuthenticationChallenge&)
@@ -224,13 +232,6 @@ void FrameLoaderClient::dispatchDidCancelAuthenticationChallenge(WebCore::Docume
     notImplemented();
 }
 
-// We convert this to string because it's easier to use strings as
-// keys in a GHashTable.
-static char* toString(unsigned long identifier)
-{
-    return g_strdup_printf("%ld", identifier);
-}
-
 void FrameLoaderClient::dispatchWillSendRequest(WebCore::DocumentLoader* loader, unsigned long identifier, ResourceRequest& request, const ResourceResponse& redirectResponse)
 {
     GRefPtr<WebKitNetworkResponse> networkResponse(0);
@@ -295,7 +296,7 @@ void FrameLoaderClient::postProgressEstimateChangedNotification()
     WebKitWebView* webView = getViewFromFrame(m_frame);
     Page* corePage = core(webView);
 
-    g_signal_emit_by_name(webView, "load-progress-changed", lround(corePage->progress()->estimatedProgress()*100));
+    g_signal_emit_by_name(webView, "load-progress-changed", lround(corePage->progress().estimatedProgress()*100));
 
     g_object_notify(G_OBJECT(webView), "progress");
 }
@@ -344,7 +345,7 @@ void FrameLoaderClient::dispatchDecidePolicyForResponse(FramePolicyFunction poli
         return;
 
     if (resourceRequest.isNull()) {
-        (core(m_frame)->loader()->policyChecker()->*policyFunction)(PolicyIgnore);
+        (core(m_frame)->loader().policyChecker()->*policyFunction)(PolicyIgnore);
         return;
     }
 
@@ -420,7 +421,7 @@ void FrameLoaderClient::dispatchDecidePolicyForNewWindowAction(FramePolicyFuncti
         return;
 
     if (resourceRequest.isNull()) {
-        (core(m_frame)->loader()->policyChecker()->*policyFunction)(PolicyIgnore);
+        (core(m_frame)->loader().policyChecker()->*policyFunction)(PolicyIgnore);
         return;
     }
 
@@ -440,7 +441,7 @@ void FrameLoaderClient::dispatchDecidePolicyForNewWindowAction(FramePolicyFuncti
     // FIXME: I think Qt version marshals this to another thread so when we
     // have multi-threaded download, we might need to do the same
     if (!isHandled)
-        (core(m_frame)->loader()->policyChecker()->*policyFunction)(PolicyUse);
+        (core(m_frame)->loader().policyChecker()->*policyFunction)(PolicyUse);
 }
 
 void FrameLoaderClient::dispatchDecidePolicyForNavigationAction(FramePolicyFunction policyFunction, const NavigationAction& action, const ResourceRequest& resourceRequest, PassRefPtr<FormState>)
@@ -450,7 +451,7 @@ void FrameLoaderClient::dispatchDecidePolicyForNavigationAction(FramePolicyFunct
         return;
 
     if (resourceRequest.isNull()) {
-        (core(m_frame)->loader()->policyChecker()->*policyFunction)(PolicyIgnore);
+        (core(m_frame)->loader().policyChecker()->*policyFunction)(PolicyIgnore);
         return;
     }
 
@@ -467,7 +468,7 @@ void FrameLoaderClient::dispatchDecidePolicyForNavigationAction(FramePolicyFunct
     g_signal_emit_by_name(webView, "navigation-requested", m_frame, request.get(), &response);
 
     if (response == WEBKIT_NAVIGATION_RESPONSE_IGNORE) {
-        (core(m_frame)->loader()->policyChecker()->*policyFunction)(PolicyIgnore);
+        (core(m_frame)->loader().policyChecker()->*policyFunction)(PolicyIgnore);
         return;
     }
 
@@ -531,8 +532,8 @@ PassRefPtr<Frame> FrameLoaderClient::createFrame(const KURL& url, const String&
     RefPtr<Frame> childFrame = Frame::create(page, ownerElement, new FrameLoaderClient(kitFrame));
     framePrivate->coreFrame = childFrame.get();
 
-    childFrame->tree()->setName(name);
-    parentFrame->tree()->appendChild(childFrame);
+    childFrame->tree().setName(name);
+    parentFrame->tree().appendChild(childFrame);
     childFrame->init();
 
     // The creation of the frame may have run arbitrary JavaScript that removed it from the page already.
@@ -541,10 +542,10 @@ PassRefPtr<Frame> FrameLoaderClient::createFrame(const KURL& url, const String&
 
     g_signal_emit_by_name(webView, "frame-created", kitFrame);
 
-    parentFrame->loader()->loadURLIntoChildFrame(url, referrer, childFrame.get());
+    parentFrame->loader().loadURLIntoChildFrame(url, referrer, childFrame.get());
 
     // The frame's onload handler may have removed it from the document.
-    if (!childFrame->tree()->parent())
+    if (!childFrame->tree().parent())
         return 0;
 
     return childFrame.release();
@@ -552,7 +553,7 @@ PassRefPtr<Frame> FrameLoaderClient::createFrame(const KURL& url, const String&
 
 void FrameLoaderClient::redirectDataToPlugin(Widget* pluginWidget)
 {
-    m_pluginView = static_cast<PluginView*>(pluginWidget);
+    m_pluginView = toPluginView(pluginWidget);
     if (pluginWidget)
         m_hasSentResponseToPlugin = false;
 }
@@ -584,14 +585,13 @@ void FrameLoaderClient::dispatchDidClearWindowObjectInWorld(DOMWrapperWorld* wor
     Frame* coreFrame = core(m_frame);
     ASSERT(coreFrame);
 
-    Settings* settings = coreFrame->settings();
-    if (!settings || !settings->isScriptEnabled())
+    if (!coreFrame->settings().isScriptEnabled())
         return;
 
     // TODO: Consider using g_signal_has_handler_pending() to avoid the overhead
     // when there are no handlers.
-    JSGlobalContextRef context = toGlobalRef(coreFrame->script()->globalObject(mainThreadNormalWorld())->globalExec());
-    JSObjectRef windowObject = toRef(coreFrame->script()->globalObject(mainThreadNormalWorld()));
+    JSGlobalContextRef context = toGlobalRef(coreFrame->script().globalObject(mainThreadNormalWorld())->globalExec());
+    JSObjectRef windowObject = toRef(coreFrame->script().globalObject(mainThreadNormalWorld()));
     ASSERT(windowObject);
 
     WebKitWebView* webView = getViewFromFrame(m_frame);
@@ -856,7 +856,7 @@ void FrameLoaderClient::dispatchDidCommitLoad(bool isNavigatingWithinPage)
 
     WebKitWebFramePrivate* priv = m_frame->priv;
     g_free(priv->uri);
-    priv->uri = g_strdup(core(m_frame)->loader()->activeDocumentLoader()->url().string().utf8().data());
+    priv->uri = g_strdup(core(m_frame)->loader().activeDocumentLoader()->url().string().utf8().data());
     g_object_notify(G_OBJECT(m_frame), "uri");
     if (!isNavigatingWithinPage) {
         g_free(priv->title);
@@ -1020,6 +1020,8 @@ void FrameLoaderClient::dispatchDidFinishLoading(WebCore::DocumentLoader* loader
     g_signal_emit_by_name(webResource, "load-finished");
     g_signal_emit_by_name(m_frame, "resource-load-finished", webResource);
     g_signal_emit_by_name(webView, "resource-load-finished", m_frame, webResource);
+
+    webkitWebViewRemoveSubresource(webView, identifierString.get());
 }
 
 void FrameLoaderClient::dispatchDidFailLoading(WebCore::DocumentLoader* loader, unsigned long identifier, const ResourceError& error)
@@ -1042,6 +1044,8 @@ void FrameLoaderClient::dispatchDidFailLoading(WebCore::DocumentLoader* loader,
     g_signal_emit_by_name(webResource, "load-failed", webError.get());
     g_signal_emit_by_name(m_frame, "resource-load-failed", webResource, webError.get());
     g_signal_emit_by_name(webView, "resource-load-failed", m_frame, webResource, webError.get());
+
+    webkitWebViewRemoveSubresource(webView, identifierString.get());
 }
 
 bool FrameLoaderClient::dispatchDidLoadResourceFromMemoryCache(WebCore::DocumentLoader*, const ResourceRequest&, const ResourceResponse&, int length)
@@ -1063,58 +1067,41 @@ void FrameLoaderClient::dispatchDidFailLoad(const ResourceError& error)
     notifyStatus(m_frame, WEBKIT_LOAD_FAILED);
 
     WebKitWebView* webView = getViewFromFrame(m_frame);
-    GError* webError = g_error_new_literal(g_quark_from_string(error.domain().utf8().data()),
-                                           error.errorCode(),
-                                           error.localizedDescription().utf8().data());
+    GOwnPtr<GError> webError(g_error_new_literal(
+        g_quark_from_string(error.domain().utf8().data()),
+        error.errorCode(),
+        error.localizedDescription().utf8().data()));
     gboolean isHandled = false;
-    g_signal_emit_by_name(webView, "load-error", m_frame, error.failingURL().utf8().data(), webError, &isHandled);
+    g_signal_emit_by_name(webView, "load-error", m_frame, error.failingURL().utf8().data(), webError.get(), &isHandled);
 
-    if (isHandled) {
-        g_error_free(webError);
+    if (isHandled || !shouldFallBack(error))
         return;
-    }
-
-    if (!shouldFallBack(error)) {
-        g_error_free(webError);
-        return;
-    }
 
     m_loadingErrorPage = true;
 
     String content;
-    gchar* fileContent = 0;
     GOwnPtr<gchar> errorPath(g_build_filename(sharedResourcesPath().data(), "resources", "error.html", NULL));
-    gchar* errorURI = g_filename_to_uri(errorPath.get(), 0, 0);
-
-    GFile* errorFile = g_file_new_for_uri(errorURI);
-    g_free(errorURI);
+    GRefPtr<GFile> errorFile = adoptGRef(g_file_new_for_path(errorPath.get()));
 
     if (!errorFile)
         content = makeString("<html><body>", webError->message, "</body></html>");
     else {
-        gboolean loaded = g_file_load_contents(errorFile, 0, &fileContent, 0, 0, 0);
-        if (!loaded)
+        GOwnPtr<gchar> fileContent;
+        if (!g_file_load_contents(errorFile.get(), 0, &fileContent.outPtr(), 0, 0, 0))
             content = makeString("<html><body>", webError->message, "</body></html>");
         else
-            content = String::format(fileContent, error.failingURL().utf8().data(), webError->message);
+            content = String::format(fileContent.get(), error.failingURL().utf8().data(), webError->message);
     }
 
     webkit_web_frame_load_alternate_string(m_frame, content.utf8().data(), 0, error.failingURL().utf8().data());
-
-    g_free(fileContent);
-
-    if (errorFile)
-        g_object_unref(errorFile);
-
-    g_error_free(webError);
 }
 
-void FrameLoaderClient::convertMainResourceLoadToDownload(MainResourceLoader* mainResourceLoader, const ResourceRequest& request, const ResourceResponse& response)
+void FrameLoaderClient::convertMainResourceLoadToDownload(WebCore::DocumentLoader* documentLoader, const ResourceRequest& request, const ResourceResponse& response)
 {
     GRefPtr<WebKitNetworkRequest> networkRequest(adoptGRef(kitNew(request)));
     WebKitWebView* view = getViewFromFrame(m_frame);
 
-    webkit_web_view_request_download(view, networkRequest.get(), response, mainResourceLoader->loader()->handle());
+    webkit_web_view_request_download(view, networkRequest.get(), response, documentLoader->mainResourceLoader()->handle());
 }
 
 ResourceError FrameLoaderClient::cancelledError(const ResourceRequest& request)
@@ -1238,7 +1225,7 @@ void FrameLoaderClient::transitionToCommittedFromCachedFrame(CachedFrame* cached
     ASSERT(cachedFrame->view());
 
     Frame* frame = core(m_frame);
-    if (frame != frame->page()->mainFrame())
+    if (frame != &frame->page()->mainFrame())
         return;
 
     postCommitFrameViewSetup(m_frame);
@@ -1248,11 +1235,7 @@ void FrameLoaderClient::transitionToCommittedForNewPage()
 {
     WebKitWebView* containingWindow = getViewFromFrame(m_frame);
     GtkAllocation allocation;
-#if GTK_CHECK_VERSION(2, 18, 0)
     gtk_widget_get_allocation(GTK_WIDGET(containingWindow), &allocation);
-#else
-    allocation = GTK_WIDGET(containingWindow)->allocation;
-#endif
     IntSize size = IntSize(allocation.width, allocation.height);
     bool transparent = webkit_web_view_get_transparent(containingWindow);
     Color backgroundColor = transparent ? WebCore::Color::transparent : WebCore::Color::white;
@@ -1262,7 +1245,7 @@ void FrameLoaderClient::transitionToCommittedForNewPage()
     frame->createView(size, backgroundColor, transparent);
 
     // We need to do further manipulation on the FrameView if it was the mainFrame
-    if (frame != frame->page()->mainFrame())
+    if (frame != &frame->page()->mainFrame())
         return;
 
     postCommitFrameViewSetup(m_frame);
